==> org.opennms.features.telemetry.common-21.1.0-20200321.055709-110-sources.jar: org/opennms/netmgt/telemetry/utils/ListenerFactory.java <==
/*******************************************************************************
 * This file is part of OpenNMS(R).
 *
 * Copyright (C) 2017-2017 The OpenNMS Group, Inc.
 * OpenNMS(R) is Copyright (C) 1999-2017 The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OpenNMS(R) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OpenNMS(R).  If not, see:
 *      http://www.gnu.org/licenses/
 *
 * For more information contact:
 *     OpenNMS(R) Licensing <license@opennms.org>
 *     http://www.opennms.org/
 *     http://www.opennms.com/
 *******************************************************************************/

package org.opennms.netmgt.telemetry.utils;

import org.opennms.core.ipc.sink.api.AsyncDispatcher;
import org.opennms.netmgt.telemetry.listeners.api.Listener;
import org.opennms.netmgt.telemetry.listeners.api.ListenerDefinition;
import org.opennms.netmgt.telemetry.listeners.api.TelemetryMessage;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.PropertyAccessorFactory;

import java.lang.reflect.Constructor;

public class ListenerFactory {

    public static Listener buildListener(ListenerDefinition listenerDef, AsyncDispatcher<TelemetryMessage> dispatcher) throws Exception {
        // Instantiate the associated class
        final Object listenerInstance;
        try {
            final Class<?> clazz = Class.forName(listenerDef.getClassName());
            final Constructor<?> ctor = clazz.getConstructor();
            listenerInstance = ctor.newInstance();
        } catch (Exception e) {
            throw new RuntimeException(String.format("Failed to instantiate listener with class name '%s'.",
                    listenerDef.getClassName(), e));
        }

        // Cast
        if (!(listenerInstance instanceof Listener)) {
            throw new IllegalArgumentException(String.format("%s must implement %s", listenerDef.getClassName(), Listener.class.getCanonicalName()));
        }
        final Listener listener = (Listener)listenerInstance;

        // Apply the parameters
        final BeanWrapper wrapper = PropertyAccessorFactory.forBeanPropertyAccess(listener);
        wrapper.setPropertyValues(listenerDef.getParameterMap());

        // Update the name
        // The one given in the definition wins over any that may be set by the parameters
        listener.setName(listenerDef.getName());

        // Use the given dispatcher
        listener.setDispatcher(dispatcher);

        return listener;
    }
}

==> org.opennms.features.telemetry.common-23.0.0-20180629.044853-21-sources.jar: org/opennms/netmgt/telemetry/utils/ListenerFactory.java <==
/*******************************************************************************
 * This file is part of OpenNMS(R).
 *
 * Copyright (C) 2017-2017 The OpenNMS Group, Inc.
 * OpenNMS(R) is Copyright (C) 1999-2017 The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OpenNMS(R) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OpenNMS(R).  If not, see:
 *      http://www.gnu.org/licenses/
 *
 * For more information contact:
 *     OpenNMS(R) Licensing <license@opennms.org>
 *     http://www.opennms.org/
 *     http://www.opennms.com/
 *******************************************************************************/

package org.opennms.netmgt.telemetry.utils;

import java.lang.reflect.Constructor;

import org.opennms.core.ipc.sink.api.AsyncDispatcher;
import org.opennms.netmgt.telemetry.listeners.api.Listener;
import org.opennms.netmgt.telemetry.listeners.api.ListenerDefinition;
import org.opennms.netmgt.telemetry.listeners.api.TelemetryMessage;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.PropertyAccessorFactory;

public class ListenerFactory {

    public static Listener buildListener(ListenerDefinition listenerDef, AsyncDispatcher<TelemetryMessage> dispatcher) throws Exception {
        // Instantiate the associated class
        final Object listenerInstance;
        try {
            final Class<?> clazz = Class.forName(listenerDef.getClassName());
            final Constructor<?> ctor = clazz.getConstructor();
            listenerInstance = ctor.newInstance();
        } catch (Exception e) {
            throw new RuntimeException(String.format("Failed to instantiate listener with class name '%s'.",
                    listenerDef.getClassName()), e);
        }

        // Cast
        if (!(listenerInstance instanceof Listener)) {
            throw new IllegalArgumentException(String.format("%s must implement %s", listenerDef.getClassName(), Listener.class.getCanonicalName()));
        }
        final Listener listener = (Listener)listenerInstance;

        // Apply the parameters
        final BeanWrapper wrapper = PropertyAccessorFactory.forBeanPropertyAccess(listener);
        wrapper.setPropertyValues(listenerDef.getParameterMap());

        // Update the name
        // The one given in the definition wins over any that may be set by the parameters
        listener.setName(listenerDef.getName());

        // Use the given dispatcher
        listener.setDispatcher(dispatcher);

        return listener;
    }
}
